/* $Id: avatarworm.cpp,v 1.59 2005/05/13 08:12:24 uwe Exp $ */

#include  "avatarworm.hpp"
#include  "audio.hpp"
#include  "random.hpp"

/**********************************************************/
// some graphic settings we cannot extract from the sprites
#define  MIN_AIMING_ANGLE     0
#define  AIMING_ANGLE_STEP    5
// derived constants
#define  NUM_AIMING_ANGLES  (( ((int)MAX_AIMING_ANGLE - MIN_AIMING_ANGLE) / \
                                AIMING_ANGLE_STEP ) + 1)
#define  NUM_SEQUENCES_PER_DIRECTION  NUM_AIMING_ANGLES

/**********************************************************/

AvatarWorm::AvatarWorm()
		  : Avatar()
{
	String sampleFilename;

	m_maxHealth      = 1000000;
	m_health         = m_maxHealth;
	m_mass           = 250.0;
	m_collRectX      =  -4;
	m_collRectY      = -15;
	m_collRectWidth  =   9;
	m_collRectHeight =  21;
	m_weaponPivotDY  =   0;

	// set the initial colors
	m_Colors[SKIN_COLOR] = TPLCOL_RGB( 255, 255, 255 );
	m_Colors[EYE_COLOR]  = TPLCOL_DONT_COL;
	m_Colors[GUN_COLOR]  = TPLCOL_RGB(   0, 100, 230 );
	// initialize the player and team color per default
	// for an individual fight
	m_Colors[PLAYER_COLOR] = m_Colors[SKIN_COLOR];
	m_Colors[TEAM_COLOR]   = m_Colors[GUN_COLOR];
	
	for( int s = 0; s < N_INJURED_SAMPLES; s++ ) {
		sampleFilename.format( "sound/avatar/worm/injured%01i.wav", s+1 );
		m_injuredSample[s] = Audio::getInstance()->loadSound( sampleFilename );
	}

	for( int s = 0; s < N_DIE_SAMPLES; s++ ) {
		sampleFilename.format( "sound/avatar/worm/die%02i.wav", s+1 );
		m_dieSample[s] = Audio::getInstance()->loadSound( sampleFilename );
	}

}

/**********************************************************/

int AvatarWorm::applyDamage( const Particles::ParticleData& p )
{
	const int damage = Avatar::applyDamage( p );

	// \todo UGLY HACK!!! parts of this function should move to class Avatar
	if( damage <= 0 )  return 0;

	LOG( 5 ) INFO( "Avatar::applyDamage: player '%s' was hit\n",
	               static_cast<char*>( getPlayer()->getName() ));
	Uint8 r, g, b;
	real brightness;

	LOG( 5 ) INFO( "Avatar::applyDamage: damage: %i => new health: %i\n",
	                damage, m_health );

	Player* const shooter = m_worldPointer->getPlayerByID( p.owner );
	if( shooter ) {
		Scorekeeper::getInstance()
			->playerDamagedPlayer( shooter->getPlayerID(),
			                       getPlayer()->getPlayerID(),
			                       damage,
			                       m_health <= 0 );
	}
		                                                

	if( m_health > 0 ) {
		// play sound
		if( m_damageSampleTimeout == 0 ) {
			Audio::getInstance()->playSound( m_injuredSample[globalRnd.getUint32() % N_INJURED_SAMPLES],
			                                 ROUND( m_positionX ), ROUND( m_positionY ) );
			m_damageSampleTimeout = 5;
		}

		// generate blood
		for( int i = 0; i < 3; i++ ) {
			brightness = 0.7 + 0.3*globalRnd.getNormedReal();
			r = ROUND( static_cast<Uint8>( m_Colors[SKIN_COLOR] >> 16 ) * brightness );
			g = ROUND( static_cast<Uint8>( m_Colors[SKIN_COLOR] >>  8 ) * brightness );
			b = ROUND( static_cast<Uint8>( m_Colors[SKIN_COLOR]       ) * brightness );

			m_worldPointer->getParticles()->addParticle( Particles::ParticleData( p.pos,
			     Vector( 6.0*(0.5 - globalRnd.getNormedReal()),
			             6.0*(0.5 - globalRnd.getNormedReal()) ) + getSpeed(),
			     Particles::BOUNCE_EARTH | Particles::STAIN | Particles::REDUNDANT, 0,
			     0, 75, 1, (((r << 8) | g) << 8) | b ));
		}
	}
	else {
		// detach all ropes
		detachFromAll();

		// play sound
		Audio::getInstance()->playSound( m_dieSample[globalRnd.getUint32() % N_DIE_SAMPLES],
		                                 ROUND( m_positionX ), ROUND( m_positionY ) );

		// generate a lot of blood
		for( int i = 0; i < 1000; i++ ) {
			brightness = 0.7 + 0.3*globalRnd.getNormedReal();
			r = ROUND( static_cast<Uint8>( m_Colors[SKIN_COLOR] >> 16 ) * brightness );
			g = ROUND( static_cast<Uint8>( m_Colors[SKIN_COLOR] >>  8 ) * brightness );
			b = ROUND( static_cast<Uint8>( m_Colors[SKIN_COLOR]       ) * brightness );

			m_worldPointer->getParticles()->addParticle( Particles::ParticleData( p.pos,
			     globalRnd.getVector( 1 + 6 * globalRnd.getNormedReal() ) + getSpeed(),
			     Particles::BOUNCE_EARTH | Particles::STAIN | Particles::REDUNDANT, 0,
			     0, 75, 1, (((r << 8) | g) << 8) | b ));
		}

		// remove avatar stencil
		removeCollRect();
		setState( VISIBLE | WEAPON_VISIBLE | LIVING | SHOOTING, false );

		// reset the bonus pack for the "sleeping state" of the avatar
		m_bonusPack.resetForRespawn();
		
		// send message
		if ( shooter ) {
			if ( getPlayer() == shooter ) {
				m_messageSink->addMessage( MessageGenerator::createSuicideMessage( getPlayer()->getName() ) );
			}
			else {
				m_messageSink->addMessage( MessageGenerator::createFragMessage( shooter->getName(), getPlayer()->getName() ) );
			}
		}
	}

	return damage;
}

/**********************************************************/

int AvatarWorm::applyDamage( const int damage, Player* shooter ) {
	const int realDamage = Avatar::applyDamage( damage, shooter );

	if( realDamage <= 0 ) return 0;

	LOG( 5 ) INFO( "Avatar::applyDamage: player '%s' was hit\n",
	               static_cast<char*>( getPlayer()->getName() ));
	Uint8 r, g, b;
	real brightness;

	LOG( 5 ) INFO( "Avatar::applyDamage: damage: %i => new health: %i\n",
	                realDamage, m_health );

	Scorekeeper::getInstance()->playerDamagedPlayer( shooter->getPlayerID(),
	                                                 getPlayer()->getPlayerID(),
	                                                 realDamage,
	                                                 m_health <= 0 );

	if( m_health > 0 ) {
		// play sound
		if( m_damageSampleTimeout == 0 ) {
			Audio::getInstance()->playSound( m_injuredSample[globalRnd.getUint32() % N_INJURED_SAMPLES],
			                                 ROUND( m_positionX ), ROUND( m_positionY ) );
			m_damageSampleTimeout = 5;
		}

		// generate blood
		for( int i = 0; i < 3; i++ ) {
			brightness = 0.7 + 0.3*globalRnd.getNormedReal();
			r = ROUND( static_cast<Uint8>( m_Colors[SKIN_COLOR] >> 16 ) * brightness );
			g = ROUND( static_cast<Uint8>( m_Colors[SKIN_COLOR] >>  8 ) * brightness );
			b = ROUND( static_cast<Uint8>( m_Colors[SKIN_COLOR]       ) * brightness );

			m_worldPointer->getParticles()->addParticle( Particles::ParticleData( getPosition(),
			     Vector( 6.0*(0.5 - globalRnd.getNormedReal()),
			             6.0*(0.5 - globalRnd.getNormedReal()) ) + getSpeed(),
			     Particles::BOUNCE_EARTH | Particles::STAIN | Particles::REDUNDANT, 0,
			     0, 75, 1, (((r << 8) | g) << 8) | b ));
		}
	}
	else {
		// detach all ropes
		detachFromAll();

		// play sound
		Audio::getInstance()->playSound( m_dieSample[globalRnd.getUint32() % N_DIE_SAMPLES],
		                                 ROUND( m_positionX ), ROUND( m_positionY ) );

		// generate a lot of blood
		for( int i = 0; i < 1000; i++ ) {
			brightness = 0.7 + 0.3*globalRnd.getNormedReal();
			r = ROUND( static_cast<Uint8>( m_Colors[SKIN_COLOR] >> 16 ) * brightness );
			g = ROUND( static_cast<Uint8>( m_Colors[SKIN_COLOR] >>  8 ) * brightness );
			b = ROUND( static_cast<Uint8>( m_Colors[SKIN_COLOR]       ) * brightness );

			m_worldPointer->getParticles()->addParticle( Particles::ParticleData( getPosition(),
			     globalRnd.getVector( 2 + 6 * globalRnd.getNormedReal() ) + getSpeed(),
			     Particles::BOUNCE_EARTH | Particles::STAIN | Particles::REDUNDANT, 0,
			     0, 75, 1, (((r << 8) | g) << 8) | b ));
		}

		// remove avatar stencil
		removeCollRect();
		setState( VISIBLE | WEAPON_VISIBLE | LIVING | SHOOTING, false );

		// reset the bonus pack for the "sleeping state" of the avatar
		m_bonusPack.resetForRespawn();
	}

	return realDamage;
}

/**********************************************************/

void AvatarWorm::updateSpriteSelection()
{
	// set the sequence index.
	// NOTE: setSequence also updates m_SequenceLength.
	setSequence( NUM_SEQUENCES_PER_DIRECTION * getDirection() +
	             (ROUND( getAimingAngle()) / AIMING_ANGLE_STEP) );

	// only reset the frame index, if we cannot continue
	// this movement in a different aiming angle
	if( getFrame() >= m_SequenceLength )  setFrame( 0 );
}

/**********************************************************/

AvatarWorm::~AvatarWorm()
{
}

/**********************************************************/
