/* $Id: client.hpp,v 1.44 2005/04/26 12:14:49 pohlt Exp $ */

#ifndef _CLIENT_HPP_
#define _CLIENT_HPP_

#include <set>
#include <SDL.h>
#include <SDL_thread.h>

#include "wopsettings.hpp"
#include "loader.hpp"
#include "video.hpp"
#include "audio.hpp"
#include "input.hpp"
#include "event.hpp"
#include "clientcomm.hpp"
#include "world.hpp"
#include "player.hpp"
#include "wopgui.hpp"
#include "wopbackground.hpp"
#include "messages.hpp"

/**********************************************************/

class Client : public MessageSink {
	private:
		Video        m_video;
		Audio*       m_audio;
		Input        m_input;
		ClientComm   m_comm;
		World       *m_world;

		ProgressLog  m_ProgressLog; //!< Logging Object for various progesses

		volatile Status m_status;

		Event          m_myEvent;

		SDLKey         m_ropeOnKey;
		
		SDL_Surface   *m_mapSurface;
		SDL_Rect       m_mapRect;
		WopBackground *m_background;
		SDL_Surface   *m_logo;
		SDL_Rect       m_logoRect;

		Player        *m_player;
		Uint8          m_frame;

		WopGUI        *m_gui;
		bool           m_showWorld;
		real           m_viewX,    m_viewY,
		               m_viewVelX, m_viewVelY;

		SDL_Thread    *m_displayThread;
		SDL_cond      *m_displayLoopCond;
		SDL_mutex     *m_displayLoopMutex;

		bool           m_doScreenshot;

		bool waitForAnyKeyPressed( const int tenthsOfSecond );


		// SDL_TimerID m_sendTimer;
		SDL_TimerID m_frameTimer;

	public:
		Client();
		virtual ~Client();
		Event getEvent() const { return m_myEvent; }
		void init();
		std::set<ServerEntry> lookForServer() {
			return m_comm.lookForServer();
		}
		bool connectToServer( ServerEntry& server );
		void disconnect();
		void processEvents();
		void updateGraphics();
		// void showSplashScreen();
		void displayLoop();
		void setFrame( Uint8 frame ) { m_frame = frame; }
		void run();
		void exit();
		void eventLoop();
		void sendEvent() { m_comm.sendEvent( m_myEvent ); }
		void addPlayer( Player* player, Uint8 frame );
		void removePlayer( Uint8 playerIndex, Uint8 frame );
		void sendChatMessage( String& message );
		void showMessage( String& message, const SDL_Color& color );
		void setStatus( Status status ) { m_status = status; }
		
		Status getStatus() const { return m_status; }
		bool handleEvent( SDL_Event* event );
		Player* getLocalPlayer( void ) { return m_player; }
		virtual void addMessage( std::string s );
	private:
		bool receiveAndProcessFrame( Uint8 frame );
};

/**********************************************************/

int displayLoopWrapper( void* data );
// Uint32 sendTimerCallback( Uint32 interval, void* data );
Uint32 frameTimerCallback( Uint32 interval, void* data );

/**********************************************************/

#endif
