/* $Id: collidableobject.cpp,v 1.24 2005/04/15 13:33:41 pohlt Exp $ */

#include "collidableobject.hpp"
#include "world.hpp"
#include "flag.hpp"

/**********************************************************/

CollidableObject::CollidableObject() :
	m_damageSampleTimeout( 0 ),
	m_collRectSet( false ),
	m_firstUpdate( true ) {
}

/**********************************************************/

CollidableObject::~CollidableObject() {
	removeCollRect();
}

/**********************************************************/

bool CollidableObject::testGrounded() {
	int dummy;

	return ! m_worldPointer->getMap()->testLinePassable(
	           ROUND( m_positionX ) + m_collRectX,
	           ROUND( m_positionY ) + m_collRectY + m_collRectHeight,
	           ROUND( m_positionX ) + m_collRectX + m_collRectWidth - 1,
	           ROUND( m_positionY ) + m_collRectY + m_collRectHeight,
	           dummy, dummy, dummy, dummy, Flags::OBSTACLE_EARTH | Flags::OBSTACLE_OBJECT );
}

/**********************************************************/

void CollidableObject::setCollRect() {
	DBG( 3 ) CHECK( ! m_collRectSet, "CollidableObject::setCollRect: collRect already set\n" );

	const int posX = ROUND( m_positionX ) + m_collRectX,
	          posY = ROUND( m_positionY ) + m_collRectY;
	LOG( 5 ) INFO( "CollidableObject::setCollRect: set at %i, %i\n", posX, posY );

	m_worldPointer->getMap()->setCollRect( posX, posY, m_collRectWidth, m_collRectHeight );
	m_collRectSet = true;
}

/**********************************************************/

void CollidableObject::removeCollRect() {
	if( ! m_collRectSet ) return;

	const int posX = ROUND( m_positionX ) + m_collRectX,
	          posY = ROUND( m_positionY ) + m_collRectY;
	LOG( 5 ) INFO( "CollidableObject::setCollRect: set at %i, %i\n", posX, posY );

	m_worldPointer->getMap()->removeCollRect( posX, posY, m_collRectWidth, m_collRectHeight );
	m_collRectSet = false;
}

/**********************************************************/

int CollidableObject::applyDamage( const Particles::ParticleData& p ) {
	// recoil by particles
	m_speedX += 2 * p.vel.x / m_mass; // factor 2 is unphysical but also more fun
	m_speedY += 2 * p.vel.y / m_mass; // factor 2 is unphysical but also more fun
	setState( GROUNDED, false );

	const real vDiff = min( (p.vel - getSpeed()).abs2(), static_cast<real>( 900.0 ));
	const int damage = min( ROUND( p.damage * vDiff ), m_health );

	LOG( 5 ) INFO( "CollidableObject::applyDamage: health: %i, damage: %i\n", m_health, damage );

	m_health -= damage;
	return damage;
}

/**********************************************************/

int CollidableObject::applyDamage( const int damage, Player* shooter ) {
	const int realDamage = min( damage, m_health );
	m_health -= realDamage;
	return realDamage;
}

/**********************************************************/

void CollidableObject::serialize( Uint8*& bufferPointer ) const {
	// expands to a check of the buffer movement
	START_OBJECT_SERIALIZED_SIZE_CHECK( bufferPointer );

	AttachableObject::serialize( bufferPointer );

	Serialize<real>::serialize( m_mass, bufferPointer );
	Serialize<Sint32>::serialize( m_health, bufferPointer );
	Serialize<Sint32>::serialize( m_damageSampleTimeout, bufferPointer );
	Serialize<bool>::serialize( m_collRectSet, bufferPointer );
	Serialize<bool>::serialize( m_firstUpdate, bufferPointer );

	// expands to tag serialization
	SERIALIZE_OBJECT_TAG( bufferPointer );
	// expands to a check of the buffer movement
	END_OBJECT_SERIALIZED_SIZE_CHECK( bufferPointer, CollidableObject );
}

/**********************************************************/

void CollidableObject::deserialize( Uint8*& bufferPointer ) {
	AttachableObject::deserialize( bufferPointer );

	Serialize<real>::deserialize( bufferPointer, m_mass );
	Serialize<Sint32>::deserialize( bufferPointer, m_health );
	Serialize<Sint32>::deserialize( bufferPointer, m_damageSampleTimeout );
	Serialize<bool>::deserialize( bufferPointer, m_collRectSet );
	Serialize<bool>::deserialize( bufferPointer, m_firstUpdate );

	// expands to tag deserialization
	DESERIALIZE_OBJECT_TAG( bufferPointer );
}

/**********************************************************/

Uint32 CollidableObject::getSerializeBufferSize() const {
	return   AttachableObject::getSerializeBufferSize()
	       + Serialize<real>::sizeOf( m_mass )
	       + Serialize<Sint32>::sizeOf( m_health )
	       + Serialize<Sint32>::sizeOf( m_damageSampleTimeout )
	       + Serialize<bool>::sizeOf( m_collRectSet )
	       + Serialize<bool>::sizeOf( m_firstUpdate )
	       // adds additional size for debugging (see serialize.hpp),
	       // but only, if the tags are used
	       PLUS_TAG_SIZE( 1 );
}

/**********************************************************/
