/* $Id: global.cpp,v 1.16 2005/04/04 18:58:38 chfreund Exp $ */

#include "global.hpp"

#include <cstdio>
#include <cstdlib>
#include <cstdarg>

#include <SDL.h>

#include "wopsettings.hpp"

/**********************************************************/

#ifdef SUPPRESS_COLORED_OUTPUT
#define BLACK(s)                 s
#define BLINKINK_BLACK_ON_RED(s) s
#define GREEN(s)                 s
#define RED(s)                   s
#else
#define BLACK(s)                 "\033[0;1m"s"\033[0m"
#define BLINKINK_BLACK_ON_RED(s) "\033[05;1;41m"s"\033[0m"
#define GREEN(s)                 "\033[32m"s"\033[0m"
#define RED(s)                   "\033[31m"s"\033[0m"
#endif

/**********************************************************/

#define VFPRINTF va_list arg;\
                 va_start( arg, format );\
                 vfprintf( logFile, format, arg );\
                 va_end( arg );

/**********************************************************/

static FILE* logFile = stderr;
static int flushCounter = 0;

/**********************************************************/

void INFO( const char* format, ... ) {
	fprintf( logFile, GREEN("INFO: ") );
	VFPRINTF
	if( ++flushCounter >= 20 ) {
		FLUSH_LOG();
	}
}

/**********************************************************/

bool CHECK( const bool test, const char* format, ... ) {
	if( ! test ) {
		fprintf( logFile, RED("WARNING: ") );
		VFPRINTF
		FLUSH_LOG();
	}
	return test;
}

/**********************************************************/

bool ASSERT( const bool test, const char* format, ... ) {
	if( test ) return true;
	fprintf( logFile, BLACK("ERROR: ") );
	VFPRINTF
	FLUSH_LOG();
	
	if ( WopSettings::getInstance()->getDebug() ) {
		abort();
	}
	else {
		exit( 1 );
	}
}

/**********************************************************/

void FLUSH_LOG() {
	fflush( logFile );
	flushCounter = 0;
}

/**********************************************************/

void REDIRECT_LOG( const char* filename ) {
	logFile = stderr;

	if( filename != NULL ) {
		logFile = fopen( filename, "w" );
		if( logFile == NULL ) {
			logFile = stderr;
			CHECK( false, "INIT_LOG: could not open log file '%s' "
			       "=> printing to stderr\n", filename );
		}
	}
}

/**********************************************************/

void makePace() {
	static int delay = DESIRED_DIFF - 10;
	static int lastTicks;
	const int currentTicks = SDL_GetTicks(),
	          diff = currentTicks - lastTicks;
	
	lastTicks = currentTicks;
	if( diff < DESIRED_DIFF ) delay++;
	else if( diff > DESIRED_DIFF && delay > 0 ) delay--;

	LOG( 5 ) INFO( "MakePace: Delay is %i\n", delay );

	SDL_Delay( delay );
}

/**********************************************************/
