/* $Id: optionswindow.cpp,v 1.4 2005/02/22 11:24:06 pohlt Exp $ */

#include "optionswindow.hpp"

#include <sstream>
#include "textlabel.h"
#include "wopsettings.hpp"

OptionsWindow::OptionsWindow( WopSettings* settings, Uint32 width, Uint32 height )
	: m_settings( settings ) {
	m_optionContainer = new WidgetComposite();
	TableLayout* optionLayout = new TableLayout( m_optionContainer, 2 );
	optionLayout->setHorizontalSpacing( 5 );
	optionLayout->setColumnStretch( 1, FILL );
	m_optionContainer->setLayout( optionLayout );
	m_optionContainer->setWidth( width );
	
	SDL_Color black;
	black.r = black.g = black.b = 0;
	
	Widget* widget = new TextLabel( "Data path" );
	widget->setColor( black );
	m_dataField = new TextField( m_settings->getData(), 200, 20 );
	m_dataField->setColor( black );
	m_dataField->addActionListener( this );
	m_labels.push_back( widget );
	// m_values.push_back( textField );
	m_optionContainer->add( widget );
	m_optionContainer->add( m_dataField );
	
	widget = new TextLabel( "Player name" );
	widget->setColor( black );
	m_nameField = new TextField( m_settings->getName(), 200, 20 );
	m_nameField->setColor( black );
	m_nameField->addActionListener( this );
	m_labels.push_back( widget );
	// m_values.push_back( m_nameField );
	m_optionContainer->add( widget );
	m_optionContainer->add( m_nameField );
	
	widget = new TextLabel( "Player color" );
	widget->setColor( black );
	Uint32 color = m_settings->getPlayerColor();
	Uint32 r = ( color >> 16 ) & 0xff;
	Uint32 g = ( color >>  8 ) & 0xff;
	Uint32 b =   color & 0xff;
	std::ostringstream ostr;
	ostr << r << "," << g << "," << b;
	m_colorField = new TextField( ostr.str(), 200, 20 );
	m_colorField->setColor( black );
	m_colorField->addActionListener( this );
	m_labels.push_back( widget );
	// m_values.push_back( textField );
	m_optionContainer->add( widget );
	m_optionContainer->add( m_colorField );
	
	widget = new TextLabel( "Team number" );
	widget->setColor( black );
	ostr.str( "" );
	ostr << m_settings->getTeamID();
	m_teamField = new TextField( ostr.str(), 200, 20 );
	m_teamField->setColor( black );
	m_teamField->addActionListener( this );
	m_labels.push_back( widget );
	// m_values.push_back( textField );
	m_optionContainer->add( widget );
	m_optionContainer->add( m_teamField );
	
	widget = new TextLabel( "Keyboard type" );
	widget->setColor( black );
	m_keyboardField = new TextField( m_settings->getKeyboard(), 200, 20 );
	m_keyboardField->setColor( black );
	m_keyboardField->addActionListener( this );
	m_labels.push_back( widget );
	// m_values.push_back( textField );
	m_optionContainer->add( widget );
	m_optionContainer->add( m_keyboardField );
	
	m_buttonRow = new WidgetComposite();
	HorizontalLayout* buttonLayout = new HorizontalLayout( m_buttonRow );
	m_buttonRow->setLayout( buttonLayout );
	
	m_saveButton = new TextButton( "Save" );
	m_saveButton->setHighlightable( true );
	m_saveButton->addActionListener( this );
	m_buttonRow->add( m_saveButton );
	buttonLayout->setStretch( m_saveButton, FILL );
	
	m_backButton = new TextButton( "Back" );
	m_backButton->setHighlightable( true );
	m_backButton->addActionListener( this );
	m_buttonRow->add( m_backButton );
	buttonLayout->setStretch( m_backButton, FILL );
	
	m_buttonRow->setWidth( width );
	
	m_content = new WidgetComposite();
	VerticalLayout* contentLayout = new VerticalLayout( m_content );
	m_content->setLayout( contentLayout );
	m_content->add( m_optionContainer );
	contentLayout->setStretch( m_optionContainer, FILL );
	m_content->add( m_buttonRow );
	m_content->setMinHeight( height - 20 );
	
	setTitle( "Options" );
	setWidget( m_content );
}

void OptionsWindow::actionPerformed( Uint32 cmd, void* source ) {
	if ( source == m_backButton && cmd == Button::BUTTON_PRESSED ) {
		fireAction( BACK_PRESSED, this );
	}
	else if ( source == m_saveButton && cmd == Button::BUTTON_PRESSED ) {
		fireAction( SAVE_PRESSED, this );
	}
	else if ( source == m_nameField ) {
		if ( cmd == TextField::EDITING_FINISHED ) {
			m_settings->setName( m_nameField->getText().c_str() );
		}
		else if ( cmd == TextField::EDITING_CANCELLED ) {
			m_nameField->setText( m_settings->getName() );
		}
	}
	else if ( source == m_dataField ) {
		if ( cmd == TextField::EDITING_FINISHED ) {
			m_settings->setData( m_dataField->getText().c_str() );
		}
		else if ( cmd == TextField::EDITING_CANCELLED ) {
			m_dataField->setText( m_settings->getData() );
		}
	}
	else if ( source == m_colorField ) {
		if ( cmd == TextField::EDITING_FINISHED ) {
			int r, g, b;
			int pos1 = 0, pos2;
			std::stringstream sstr;
			pos2 = m_colorField->getText().find( ",", pos1 );
			sstr << m_colorField->getText().substr( pos1, pos2-pos1 );
			sstr >> r;
			pos1 = pos2 + 1;
			pos2 = m_colorField->getText().find( ",", pos1 );
			std::stringstream sstr2;
			sstr2 << m_colorField->getText().substr( pos1, pos2-pos1 );
			sstr2 >> g;
			pos1 = pos2 + 1;
			pos2 = m_colorField->getText().find( ",", pos1 );
			std::stringstream sstr3;
			sstr3 << m_colorField->getText().substr( pos1 );
			sstr3 >> b;
			m_settings->setPlayerColor( r << 16 | g << 8 | b );
		}
		else if ( cmd == TextField::EDITING_CANCELLED ) {
			Uint32 color = m_settings->getPlayerColor();
			Uint32 r = ( color >> 16 ) & 0xff;
			Uint32 g = ( color >>  8 ) & 0xff;
			Uint32 b =   color & 0xff;
			std::ostringstream ostr;
			ostr << r << "," << g << "," << b;
			m_colorField->setText( ostr.str() );
		}
	}
	else if ( source == m_teamField ) {
		if ( cmd == TextField::EDITING_FINISHED ) {
			std::stringstream sstr;
			int team;
			sstr << m_teamField->getText();
			sstr >> team;
			m_settings->setTeamID( team );
		}
		else if ( cmd == TextField::EDITING_CANCELLED ) {
			std::ostringstream ostr;
			ostr << m_settings->getTeamID();
			m_teamField->setText( ostr.str() );
		}
	}
	else if ( source == m_keyboardField ) {
		if ( cmd == TextField::EDITING_FINISHED ) {
			m_settings->setKeyboard( m_keyboardField->getText().c_str() );
		}
		else if ( cmd == TextField::EDITING_CANCELLED ) {
			m_keyboardField->setText( m_settings->getKeyboard() );
		}
	}
}

