/* $Id: server.hpp,v 1.9 2004/09/22 14:32:43 chfreund Exp $ */

#ifndef _SERVER_HPP_
#define _SERVER_HPP_

#include <SDL.h>

#include "sprite.hpp"
#include "wopsprites.hpp"
#include "gamereplay.hpp"

/**********************************************************/

class World;
class ServerComm;
class SDL_cond;

typedef WopSprites<SpriteDummy> ServerWopSprites;

/**********************************************************/

//! server class for WoP

/*! Server class for WoP. <br>
 *  <b>The Dummy Sprite Object</b><br>
 *  The server class has a member of type WopSprites<SpriteDummy>.
 *  This is a sprite graphic object, that contains not a single
 *  image, but all the structur information of the real sprite object,
 *  that lives only on a client, in contrary. Besides that it will
 *  never draw an image, it behaves exacly like the object on the
 *  client and is therefore built in the same way from the graphic's
 *  configuration files in the data path. This class allows the
 *  server to keep all objects in the server world synchronized with
 *  those living in a client's world, since it contains for example
 *  the important information about animation sequences lenght.
 */
class Server
{
	private:

		//! 
		void updateGraphics();
		
		World*            m_world;
		ServerComm*       m_comm;
		Uint8             m_frame;
		Uint32            m_realFrame;
		SDL_mutex*        m_syncMutex;
		SDL_cond*         m_syncCond;
		ServerWopSprites  m_sprites;   //!< dummy sprite graphic class
		GameReplay*       m_replay;
		
	public:
	
		Server();
		~Server();
		void run();
		Uint8 getFrame() { return m_frame; }
		SDL_mutex* getSyncMutex() { return m_syncMutex; }
		SDL_cond* getSyncCond() { return m_syncCond; }
		void processEvents();
		GameReplay* getReplay() { return m_replay; }
};

int serverRunWrapper( void* data );

/**********************************************************/

#endif // _SERVER_HPP_
