/* $Id: smoke.cpp,v 1.7 2005/01/13 17:29:55 uwe Exp $ */

#include "global.hpp"
#include "spriteset.hpp"
#include "world.hpp"
#include "smoke.hpp"

/**********************************************************/

Smoke::Smoke( const Sint32 size )
     : m_Size(SMALLEST_SMOKE)
{
	// sequence is chosen randomly in setSize()
	if( m_worldPointer->getSpriteInterface() ) setSize( size );
	
	m_Colors[SMOKE_COLOR] = TPLCOL_RGB(255,255,255);
}

/**********************************************************/

Smoke::~Smoke()
{
}

/**********************************************************/

void Smoke::setSize( const Sint32 size )
{
	DBG( 1 ) ASSERT( getMinSize() <= size && size <= getMaxSize(),
	                 "Smoke::setSize: size %d is out of range [%d,%d]\n",
	                 size, getMinSize(), getMaxSize() );

	// set new size
	m_Size = size;

////////////////////////////////////////////////
//// commented out this check on 27/06/2004 ////
//// I think, we do not need it ////////////////
//	if(      m_size < getMinSize() )  m_size = getMinSize();
//	else if( m_size > getMaxSize() )  m_size = getMaxSize();
////////////////////////////////////////////////
	
	// choose a sequence randomly
	// this also sets m_SequenceLength
	setSequence( globalRnd.getUint32() %
	             m_worldPointer->getSpriteInterface()->getNumSequences(*this) );
	// start sequence in any case
	m_Frame       = 0;
	m_StepCounter = 0;
}

/**********************************************************/

void Smoke::update()
{
	if( ++m_StepCounter >= FRAMES_PER_STEP ) {
		m_StepCounter = 0;
		// smoke only live for one cycle
		if( ++m_Frame >= m_SequenceLength ) {
			m_worldPointer->deleteObject( this );
			// DO NOT TOUCH OBJECT AFTER deleteObject
			return;
		}
	}
}

/**********************************************************/

Uint32 Smoke::getSerializeBufferSize() const
{
	return   NonCollidableObject::getSerializeBufferSize()
	       + NUM_COLORS * Serialize<Uint32>::sizeOf( m_Colors[0] )
	       + Serialize<Sint32>::sizeOf( m_Size )
	       + Serialize<int>::sizeOf( m_StepCounter )
	         // adds additional size for debugging (see serialize.hpp),
	         // but only, if the tags are used
	         PLUS_TAG_SIZE( 1 );
}

/**********************************************************/

void Smoke::serialize( Uint8*& bufferPointer ) const
{
	// expands to a check of the buffer movement
	START_OBJECT_SERIALIZED_SIZE_CHECK( bufferPointer );

	NonCollidableObject::serialize( bufferPointer );

	for( int i = 0; i < NUM_COLORS; i++ ) {
		Serialize<Uint32>::serialize( m_Colors[i], bufferPointer );
	}
	Serialize<Sint32>::serialize( m_Size       , bufferPointer );
	Serialize<int   >::serialize( m_StepCounter, bufferPointer );

	// expands to tag serialization
	SERIALIZE_OBJECT_TAG( bufferPointer );
	// expands to a check of the buffer movement
	END_OBJECT_SERIALIZED_SIZE_CHECK( bufferPointer, Smoke );
}

/**********************************************************/

void Smoke::deserialize( Uint8*& bufferPointer )
{
	NonCollidableObject::deserialize( bufferPointer );

	for( int i = 0; i < NUM_COLORS; i++ ) {
		Serialize<Uint32>::deserialize( bufferPointer, m_Colors[i] );
	}
	Serialize<Sint32>::deserialize( bufferPointer, m_Size		 );
	Serialize<int   >::deserialize( bufferPointer, m_StepCounter );

	// expands to tag deserialization
	DESERIALIZE_OBJECT_TAG( bufferPointer );
}

/**********************************************************/
